//*** This JavaScript library is copyright 2002 by Gavin Kistner and Refinery, Inc.
//*** Re-use and modification require permission
//*** http://www.refinery.com/
//*** mailto:gavin@refinery.com
//*** v1.2.2 -- fixed bug when autoincrement was true but nextlinenum was ommitted
//*** v1.2.1 -- maintains selected state for single (not multi) selects in the dummyrow
//*** v1.2   -- dummyrow is now cloned and html removed if "growable" is present.
//              Table now only adds blank first line if the table property "addrowonload" is set to true.
//*** v1.1   -- more robust loading attempts
//*** v1.0.1 -- fixed row add test when maxlines was not present

//ToDo: autogrow

debugLevel=0;

if (window && window.attachEvent) window.attachEvent('onload',MakeTablesDynamic);
else if (document && document.body){
	if (document.body.attachEvent) document.body.attachEvent('onload',MakeTablesDynamic);
	else document.body.onload=MakeTablesDynamic;
} else MakeTablesDynamic();

function MakeTablesDynamic(){
	//DebugOut("MakeTablesDynamic()",1);
	if (!document || (document.readyState && document.readyState!="complete")){
		setTimeout(MakeTablesDynamic,100);
		return;
	}

	var allTables = document.getElementsByTagName("TABLE");

	for (var i=0,len=allTables.length;i<len;i++){
		var table = allTables[i];

		if (table.className.indexOf("dynamic")==-1) continue;

		table.dynamic = true;
		table.deleteable = (table.className.indexOf("deleteable")!=-1);
		table.growable = (table.className.indexOf("growable")!=-1);
		table.autogrow = (table.className.indexOf("autogrow")!=-1);
		table.hideDeletedRows = table.getAttribute("hidedeletedrows") && table.getAttribute("hidedeletedrows")!='false' && table.getAttribute("hidedeletedrows")!='no';

		var onaddrow = table.getAttribute("onaddrow");
		if (onaddrow && onaddrow.length>0) table.onaddrow=window[onaddrow];

		var onbeforeaddrow = table.getAttribute("onbeforeaddrow");
		if (onbeforeaddrow && onbeforeaddrow.length>0) table.onbeforeaddrow=window[onbeforeaddrow];

		var onbeforedeleterow = table.getAttribute("onbeforedeleterow");
		if (onbeforedeleterow && onbeforedeleterow.length>0) table.onbeforedeleterow=window[onbeforedeleterow];

		var ondeleterow = table.getAttribute("ondeleterow");
		if (ondeleterow && ondeleterow.length>0) table.ondeleterow=window[ondeleterow];

		if (table.getAttribute("autoincrement")=="true"){
			table.autoIncrement = true;
			table.nextLineNum = table.getAttribute("nextlinenum")?table.getAttribute("nextlinenum")*1:table.tBodies[0].childNodes.length;
		}

		if (table.growable) DynTable_FindDummyRow(table);
		if (table.getAttribute("addrowonload") && table.getAttribute("addrowonload")=="true") DynTable_AddRow(table,true);
	}
}

function DynTable_FindDummyRow(table){
	var dummyRow = FindChildWithClass(table,'dummyrow');
	if (!dummyRow) return DebugOut("ERROR: DynTable Library couldn't find a tr.dummyrow",0);
	table.dummyRow = dummyRow.cloneNode(true);

	var dummySels = dummyRow.getElementsByTagName("SELECT");
	if (dummySels){
		var newSels = table.dummyRow.getElementsByTagName("SELECT");
		for (var i=0,len=dummySels.length;i<len;i++) newSels[i].selectedIndex = dummySels[i].selectedIndex;
	}

	dummyRow.removeNode(true);
	return table.dummyRow;
}

function DynTable_AddRow(table,autoAdd){
	//DebugOut("DynTable_AddRow("+table+","+autoAdd+")",1);
	if (!table) table=event.srcElement;
	while (table && !table.dynamic && table.tagName!="TABLE") table=table.parentElement;
	if (!table) return DebugOut("ERROR: DynTable_AddRow was not passed an HTML element inside (or being) a dynamic table",0);

	if (!autoAdd && table.onbeforeaddrow) if (table.onbeforeaddrow(table)==false) return;

	var dummyRow = table.dummyRow;
	if (!dummyRow) dummyRow = DynTable_FindDummyRow(table);
	if (!dummyRow) return DebugOut("ERROR: DynTable_AddRow couldn't find a tr.dummyrow",0);

	var tbody = table.tBodies[0];

	DynTable_AssertVisibleRows(table);
	if (table.getAttribute('maxlines')>0 && table.numVisibleRows>=table.getAttribute('maxlines')) return //DebugOut("DynTable_AddRow bailing due to too many rows",2);

	var tr = dummyRow.cloneNode(true);
	tr.className = tr.className.replace(/ ?dummyrow/,"");
	var dummySels = dummyRow.getElementsByTagName("SELECT");
	if (dummySels){
		var newSels = tr.getElementsByTagName("SELECT");
		for (var i=0,len=dummySels.length;i<len;i++) newSels[i].selectedIndex = dummySels[i].selectedIndex;
	}
	if (table.autoIncrement){
		var tds = tr.childNodes;
		var newRowNum = table.nextLineNum++;
		tr.rowNum = newRowNum;
		for (var i=0,len=tds.length;i<len;i++) if (tds[i].nodeType==1) tds[i].innerHTML = tds[i].innerHTML.replace(/_xx_/g,newRowNum);
	}

	tbody.appendChild(tr);
	table.numVisibleRows++;

	if (!autoAdd && table.onaddrow) table.onaddrow(tr,tr.rowNum,table);
}

function DynTable_DeleteRow(tr){
	//DebugOut("DynTable_DeleteRow()",1);

	if (!tr) tr=event.srcElement;
	while (tr && tr.tagName!="TR" || !tr.tagName) tr=tr.parentElement;
	if (!tr) return DebugOut("ERROR: DynTable_DeleteRow was unable to find an appropriate row to delete",0);

	var table = tr.parentElement;
	while (table && table.tagName!="TABLE" || !table.tagName) table=table.parentElement;
	if (!table) return DebugOut("ERROR: DynTable_DeleteRow was able to find a row, but no parent table (how bizarre!)",0);

	if (table.onbeforedeleterow) if (table.onbeforedeleterow(tr,tr.rowNum)==false) return;

	if (table.hideDeletedRows) tr.style.display="none";
	else tr.removeNode(true);

	DynTable_AssertVisibleRows(table);
	table.numVisibleRows--;

	if (table.growable && table.autogrow && table.numVisibleRows==0) DynTable_AddRow(table,true);

	if (table.ondeleterow){
		if (table.hideDeletedRows) table.ondeleterow(tr,tr.rowNum,table);
		else table.ondeleterow(table);
	}
}

function DynTable_AssertVisibleRows(table){
	if (table.numVisibleRows!=null) return;

	var trs = table.tBodies[0].childNodes;
	var numVisibleRows=0;
	for (var i=0,len=trs.length;i<len;i++) if (trs[i].style.display!='none' && trs[i]!=table.dummyRow) numVisibleRows++;
	table.numVisibleRows = numVisibleRows;
}