/*************************************************
HTMLWYSIWYG textarea Library
->Relies on generic_library.js for AttachEvent(), DebugOut(), and Ancestor()<-
Copyright 2002 Gavin Kistner and Refinery, Inc. -- http://www.refinery.com/
Re-use and modification prohibited without prior written consent

v1.0   2002-Jul-19  Initial release (Conduant Admin)
v1.1   2002-Jul-29  Made more flexible as generic library
v1.2   2002-Oct-16  Added ul and table buttons
v1.2.1 2002-Oct-21  Fixed so ALL buttons use buttonImagesPath
v1.2.2 2002-Oct-28  IE5.5 empty textarea display bug workaround hack   (Stupid MS)
v1.2.5 2002-Nov-08  HREFs no longer turn absolute after wysiwyg toggle (Stupid MS)
v1.2.6 2003-Jan-30  Further fix from 1.2.5 to allow whitespace between 'href', '=', and the content.
v1.2.7 2003-Feb-03  Further fix from 1.2.5 to handle & in the href
**************************************************/
//debugLevel=0;

var buttons={
	togglemode:{title:"Edit HTML",img:'html'},
	color:{title:"Choose Color",img:'color',reqSel:true},
	bold:{title:"Bold",img:'bold',reqSel:true},
	italic:{title:"Italic",img:'italic',reqSel:true},
	underline:{title:"Underline",img:'underline',reqSel:true},
	link:{title:"Insert Link",img:'link',reqSel:true},
	img:{title:"Insert Image",img:'img',reqSel:true},
	outdent:{title:"Outdent Selection",img:'outdent',reqSel:true},
	indent:{title:"Indent Selection",img:'indent',reqSel:true},
	justifyleft:{title:"Left Justify Selection",img:'justifyleft',reqSel:true},
	justifycenter:{title:"Center Justify Selection",img:'justifycenter',reqSel:true},
	justifyright:{title:"Right Justify Selection",img:'justifyright',reqSel:true},
	ul:{title:"Bulleted List",img:'ul',reqSel:true},
	tblins:{title:"Add Table",img:'tblins',reqFocus:true},
	tblrowins:{title:"Add Row",img:'tblrowins',reqTable:true},
	tblrowdel:{title:"Remove Row",img:'tblrowdel',reqTable:true},
	tblcolins:{title:"Add Column",img:'tblcolins',reqTable:true},
	tblcoldel:{title:"Remove Column",img:'tblcoldel',reqTable:true},
	tblcolright:{title:"Widen Column",img:'columnsright',reqTable:true},
	tblcolleft:{title:"Shrink Column",img:'columnsleft',reqTable:true},
	tblborders:{title:"Toggle Table Borders",img:'toggleborders',reqTable:true}
};
var HTMLWYSIWYG_currentSelection={};
var HTMLWYSIWYG_editors=[];
function HTMLWYSIWYG_Initialize(){
	//DebugOut("HTMLWYSIWYG_Initialize()",2);
	var imagesPath;
	var textareas = document.getElementsByTagName('textarea');
	if (document.body.getAttribute==null || document.body.contentEditable==null) return;

	for (var i=0,len=textareas.length;i<len;i++){
		var ta = textareas[i];
		if (!ta.getAttribute("htmlwysiwyg") || ta.getAttribute("htmlwysiwyg").toLowerCase()!='true') continue;
		
		var buttonbar = document.createElement('div');
		buttonbar.className='buttonbar';

		var buttonList = ta.getAttribute('buttonList');
		if (buttonList){
			imagesPath = ta.getAttribute('buttonImagesPath')?ta.getAttribute('buttonImagesPath'):"images/";
			var buttonGroups = buttonList.split(/{|}/);
			for (var j=0,len2=buttonGroups.length;j<len2;j++){
				var buttonGroup = document.createElement('span');
				buttonGroup.className='buttongroup';
				var buttonGroupButtons = buttonGroups[j].split(/[^a-z]/);
				for (var k=0,len3=buttonGroupButtons.length;k<len3;k++){
					var btnid = buttonGroupButtons[k];
					if (!buttons[btnid]){
						var validButtons="";
						for (var x in buttons) validButtons+=x+" - "+buttons[x].title+"\n";
						DebugOut('Invalid button "'+btnid+'" found in buttonList for textarea.name="'+ta.name+'".\nValid buttons are:\n'+validButtons,0);
						continue;
					}
					var btn = document.createElement('button');
					btn.onclick=btn.ondblclick=HTMLWYSIWYG_PerformButton;
					btn.id="btn_"+(btn.btnid=btnid);
					btn.title = buttons[btnid].title;
					if (btnid=='togglemode') btn.style.width='auto';
					var img = document.createElement('img');
					img.src=imagesPath+(buttons[btnid].img)+".gif";
					btn.appendChild(img);
					buttonGroup.appendChild(btn);
				}
				buttonbar.appendChild(buttonGroup);
			}
		}

		ta.parentNode.insertBefore(buttonbar,ta);
		ta.className='htmlwysiwyg_textarea';

		var editor = document.createElement('div');
		editor.contentEditable=!ta.readOnly && !ta.disabled;
		editor.style.width=ta.offsetWidth+"px";
		editor.style.height=ta.offsetHeight+"px"
		editor.className="htmlwysiwyg_editor"
		ta.parentNode.insertBefore(editor,ta);

		editor.onkeyup=HTMLWYSIWYG_MirrorTA;
		ta.onkeyup=HTMLWYSIWYG_MirrorEditor;

		buttonbar.style.width=ta.offsetWidth+"px";
		buttonbar.imagesPath=imagesPath;

		buttonbar.editor = editor;
		buttonbar.textarea = ta;
		editor.textarea=ta;
		editor.buttonbar=buttonbar;
		ta.buttonbar=buttonbar;
		ta.editor=editor;
		ta.style.display="none";

		HTMLWYSIWYG_MirrorEditor(ta);

		editor.onfocus=HTMLWYSIWYG_UpdateButtonBars;
		//editor.onblur=HTMLWYSIWYG_UpdateButtonBars;
		editor.onmouseup=HTMLWYSIWYG_UpdateButtonBars;

		HTMLWYSIWYG_editors[HTMLWYSIWYG_editors.length]=editor;
	}
	HTMLWYSIWYG_UpdateButtonBars();
	document.onclick=document.ondblclick=document.onkeyup=HTMLWYSIWYG_UpdateButtonBars;
}

function HTMLWYSIWYG_MirrorTA(editor){
	//DebugOut("HTMLWYSIWYG_MirrorTA("+editor+")",3);
	if (!editor){
		editor=event.srcElement;
		editor=Ancestor(editor,'div','className','htmlwysiwyg_editor');
	}
	editor.textarea.value = editor.innerHTML;
}

function HTMLWYSIWYG_MirrorEditor(ta){
	//DebugOut("HTMLWYSIWYG_MirrorEditor("+ta+")",3);
	if (!ta) ta=event.srcElement;
	ta.editor.innerHTML = ta.value;
	var srcAnchors=ta.value.match(/<a[^>]+href\s*=\s*["']?[^ >"']+/gi);
	var destAnchors=ta.editor.getElementsByTagName('a');
	
	for (var id=0,is=0,len=destAnchors.length;id<len;id++){
		if (destAnchors[id].href=='') continue;
		srcAnchors[is++].match(/href\s*=\s*["']?(.*)/i); //"this comment for syntax coloring only
		destAnchors[id].href=RegExp.$1.replace(/&amp;/g,"&");
	}
}

function HTMLWYSIWYG_PerformButton(btn){
	//DebugOut("HTMLWYSIWYG_PerformButton("+btn+")",2);
	if (!btn) btn=Ancestor(event.srcElement,'button');
	var buttonbar=Ancestor(btn,'div','className','buttonbar');
	var btnid = btn.getAttribute('btnid');
	switch(btnid){
		case 'togglemode':   	HTMLWYSIWYG_ToggleView(btn,buttonbar); break;

		case 'color':           document.execCommand('ForeColor',false,showModalDialog('colorpicker.html',null,'dialogWidth:415px;dialogHeight:474px;help:no;status:no;scroll:no')); break;
		case 'bold':            document.execCommand('Bold'); break;
		case 'italic':          document.execCommand('Italic'); break;
		case 'underline':       document.execCommand('Underline'); break;
		
		case 'link':            document.execCommand('CreateLink',true); /*alert(HTMLWYSIWYG_currentSelection.parentElement.outerHTML);*/ break;
		case 'img':             document.execCommand('InsertImage',true); break;

		case 'outdent':         document.execCommand('Outdent'); break;
		case 'indent':          document.execCommand('Indent'); break;

		case 'ul':              document.execCommand('InsertUnorderedList'); break;

		case 'justifyleft':     document.execCommand('JustifyLeft'); break;
		case 'justifycenter':   document.execCommand('JustifyCenter'); break;
		case 'justifyright':    document.execCommand('JustifyRight'); break;
		
		case 'tblins':          HTMLWYSIWYG_InsertTable(); break;
		case 'tblrowins':       HTMLWYSIWYG_InsertRow(); break;
		case 'tblrowdel':       HTMLWYSIWYG_DeleteRow(); break;
		case 'tblcolins':       HTMLWYSIWYG_InsertCol(); break;
		case 'tblcoldel':       HTMLWYSIWYG_DeleteCol(); break;
		case 'tblcolright':     HTMLWYSIWYG_SizeCol(1); break;
		case 'tblcolleft':      HTMLWYSIWYG_SizeCol(-1); break;
		case 'tblborders':      HTMLWYSIWYG_ToggleTableBorders(); break;
	}
	btn.blur();
	event.cancelBubble=true;
	HTMLWYSIWYG_MirrorTA(buttonbar.editor);
}

function HTMLWYSIWYG_ToggleView(btn,buttonbar){
	//DebugOut("HTMLWYSIWYG_ToggleView("+btn+","+buttonbar+")",2);
	var wasHTML = buttonbar.textarea.style.display!="none";
	buttonbar.textarea.style.display=wasHTML?"none":"";
	buttonbar.editor.style.display=wasHTML?"":"none";
	btn.childNodes[0].src=buttonbar.imagesPath+(wasHTML?"html.gif":"text.gif");
	btn.title=wasHTML?"Edit HTML":"Edit Visually";
	if (!buttonbar.textarea.id || buttonbar.textarea.id=="") buttonbar.textarea.id=buttonbar.textarea.uniqueID;
	if (!wasHTML) setTimeout(new Function("document.getElementById('"+buttonbar.textarea.id+"').value+=''"),50);
}
function HTMLWYSIWYG_UpdateButtonBars(){
	//DebugOut("HTMLWYSIWYG_UpdateButtonBars()",2);
	for (var i=0,len=HTMLWYSIWYG_editors.length;i<len;i++){
		HTMLWYSIWYG_ButtonsEnabled(HTMLWYSIWYG_editors[i].buttonbar);
	}
}

function HTMLWYSIWYG_ButtonsEnabled(buttonbar){
	//DebugOut("HTMLWYSIWYG_ButtonsEnabled("+buttonbar+")",3);
	HTMLWYSIWYG_UpdateSelection();
	var btns = buttonbar.getElementsByTagName('button');
	for (var i=0,len=btns.length;i<len;i++){
		var btn = btns[i];
		var btnid = btn.getAttribute('btnid');
		var btnAttr = buttons[btnid];
		btn.disabled=(btnAttr.reqFocus&&HTMLWYSIWYG_currentSelection.editor!=buttonbar.editor) || (btnAttr.reqTable && !HTMLWYSIWYG_currentSelection.parentTable) || (btnAttr.reqSel && (HTMLWYSIWYG_currentSelection.editor!=buttonbar.editor || !HTMLWYSIWYG_currentSelection.textSelected));
		btn.className=btn.disabled?"disabled":"";
		if (btn.disabled) btn.title = btn.title.replace(/( \(disabled\))?$/," (disabled)");
		else btn.title = btn.title.replace(/( \(disabled\))?$/,"");
	}
	if (event) event.cancelBubble=true;
}

function HTMLWYSIWYG_UpdateSelection(){
	//DebugOut("HTMLWYSIWYG_UpdateSelection()",3);
	var range = document.selection.createRange();
	var el = range.parentElement?range.parentElement():null;
	HTMLWYSIWYG_currentSelection.parentElement = el;
	HTMLWYSIWYG_currentSelection.editor = Ancestor(el,null,"contentEditable","true");
	HTMLWYSIWYG_currentSelection.editableFocused = HTMLWYSIWYG_currentSelection.editor!=null;
	HTMLWYSIWYG_currentSelection.textSelected = range.compareEndPoints?range.compareEndPoints("StartToEnd",range)!=0:false;
	HTMLWYSIWYG_currentSelection.range = range;
}

function HTMLWYSIWYG_InsertTable(){
	//DebugOut("HTMLWYSIWYG_InsertTable()",3);
	var t,tb,tr,th,td;
	var defaultCols=2,defaultRows=2;
	t = document.createElement("TABLE");
	t.setAttribute('edittable','true');
	t.cellSpacing="0";
	t.cellPadding="2";
	t.border="1";
	tb = t.appendChild(document.createElement("TBODY"));
	for (var i=0;i<defaultRows;i++){
		tr = tb.appendChild(document.createElement("TR"));
		for (var j=0;j<defaultCols;j++){
			if (i==0) tr.appendChild(document.createElement("TH")).width="20";
			else tr.appendChild(document.createElement("TD"));
		}
	}
	HTMLWYSIWYG_currentSelection.parentElement.insertBefore(t);
}

function HTMLWYSIWYG_InsertRow(){
	//DebugOut("HTMLWYSIWYG_InsertRow()",3);
	var t,tb,oldTR,newTR;
	if (!(t=HTMLWYSIWYG_currentSelection.parentTable)) return;
	oldTR=Ancestor(HTMLWYSIWYG_currentSelection.parentElement,'tr');
	tb=oldTR.parentNode;
	newTR=tb.insertBefore(document.createElement('tr'),oldTR.nextSibling?oldTR.nextSibling:null);
	for (var i=0,len=oldTR.childNodes.length;i<len;i++) newTR.appendChild(document.createElement('TD'));
}

function HTMLWYSIWYG_DeleteRow(){
	//DebugOut("HTMLWYSIWYG_DeleteRow()",3);
	var t,tb,oldTR;
	if (!(t=HTMLWYSIWYG_currentSelection.parentTable)) return;
	oldTR=Ancestor(HTMLWYSIWYG_currentSelection.parentElement,'tr');
	tb=oldTR.parentNode;
	tb.removeChild(oldTR);
	if (tb.childNodes.length==0) t.removeNode(true);
	HTMLWYSIWYG_ButtonsEnabled(HTMLWYSIWYG_currentSelection.editor.buttonbar);
}

function HTMLWYSIWYG_InsertCol(){
	//DebugOut("HTMLWYSIWYG_InsertCol()",3);
	var t,tb,oldTR,oldTD,newTD;
	if (!(t=HTMLWYSIWYG_currentSelection.parentTable)) return;
	oldTD=Ancestor(HTMLWYSIWYG_currentSelection.parentElement,'td');
	if (!oldTD) oldTD=Ancestor(HTMLWYSIWYG_currentSelection.parentElement,'th');
	oldTR=oldTD.parentElement;
	tb=t.tBodies[0];
	var rowCt=tb.childNodes.length;
	var colCt=oldTR.childNodes.length;
	for (var colNum=0,len=colCt;colNum<len;colNum++) if (oldTR.childNodes[colNum]==oldTD) break;
	for (var i=0,len=rowCt;i<len;i++){
		newTD=tb.childNodes[i].insertBefore(document.createElement(i==0?'TH':'TD'),colNum==(colCt-1)?null:tb.childNodes[i].childNodes[colNum+1]);
		if (i==0) newTD.width="20";
	}
}

function HTMLWYSIWYG_DeleteCol(){
	//DebugOut("HTMLWYSIWYG_DeleteCol()",3);
	var t,tb,oldTR,oldTD;
	if (!(t=HTMLWYSIWYG_currentSelection.parentTable)) return;
	oldTD=Ancestor(HTMLWYSIWYG_currentSelection.parentElement,'td');
	if (!oldTD) oldTD=Ancestor(HTMLWYSIWYG_currentSelection.parentElement,'th');
	oldTR=oldTD.parentElement;
	tb=t.tBodies[0];
	var rowCt=tb.childNodes.length;
	var colCt=oldTR.childNodes.length;
	if (colCt==1) t.removeNode(true);
	else{
		for (var colNum=0,len=colCt;colNum<len;colNum++) if (oldTR.childNodes[colNum]==oldTD) break;
		for (var i=0,len=rowCt;i<len;i++) tb.childNodes[i].childNodes[colNum].removeNode(true);
	}
	HTMLWYSIWYG_ButtonsEnabled(HTMLWYSIWYG_currentSelection.editor.buttonbar);

}

function HTMLWYSIWYG_SizeCol(inc){
	//DebugOut("HTMLWYSIWYG_SizeCol()",3);
	var t,tb,tr,td,th,increment=20;
	if (!(t=HTMLWYSIWYG_currentSelection.parentTable)) return;
	th=Ancestor(HTMLWYSIWYG_currentSelection.parentElement,'th');
	if (!th){
		tb=t.tBodies[0];
		var colCt=(tr=(td=Ancestor(HTMLWYSIWYG_currentSelection.parentElement,'td')).parentElement).childNodes.length;
		for (var colNum=0,len=colCt;colNum<len;colNum++) if (tr.childNodes[colNum]==td) break;
		th=tb.childNodes[0].childNodes[colNum];
	}
	var newWidth;
	th.width=(newWidth=((th.width==""?0:(th.width*1)))+inc*increment)>0?newWidth:'';
}

function HTMLWYSIWYG_ToggleTableBorders(){
	//DebugOut("HTMLWYSIWYG_ToggleTableBorders()",3);
	var t;
	if (!(t=HTMLWYSIWYG_currentSelection.parentTable)) return;
	t.border=(t.border==0)?1:0;
}

AttachEvent(window,'load',HTMLWYSIWYG_Initialize);

