/************************************************************
Sortable Tables Library
Copyright  2002 Gavin Kistner and Refinery, Inc. -- http://www.refinery.com
Reuse and modification permitted provided the previous line is included in full
v 1.1    20020731 -- initial librarization
v 1.2    20020805 -- added defaultSort option for th
v 1.3    20020822 -- slam rows post-sort to have 'row0' or 'row1' classes assigned
v 1.5    20030326 -- added 'ischild' attribute for paired rows; added 'price' type
v 1.5.1  20030527 -- number comparisons treat NaN as 0 now.
v 1.5.2  20030618 -- fixed AttachEvent calls to work for Safari
v 1.5.3  20030729 -- rows may be class="ischild" in addition to ischild="true"
v 1.5.4  20030801 -- date comparisons also treat NaN as 0; TH no longer select when multiply clicked
*************************************************************/

//debugLevel=1;

var SortTables_sortHead,SortTables_colToSort,SortTables_compareMethod,SortTables_sortAscending,SortTables_GetVal;
var SortTables_sortTypes = { textual:1, numeric:2, date:3, time:4, text:1, number:2, price:5 };

function SortTables_Init(){
	//DebugOut('SortTables_Init()',1);
	var tables = document.getElementsByTagName('table');
	for (var i=0,len=tables.length;i<len;i++){
		var t = tables[i];
		var tRef = VirtualPointer.create(t);
		if (!HasClass(t,"sortable")) continue;

		var sortBody = t.getAttribute("sortBody");
		if (sortBody && sortBody!="") sortBody=document.getElementById(sortBody);
		else sortBody = t.tBodies[0];
		
		var sortBodyRef = VirtualPointer.create(sortBody);
		
		var ths = t.tHead.getElementsByTagName('th');
		
		for (var j=0,len2=ths.length;j<len2;j++){
			var th = ths[j];
			var thRef = VirtualPointer.create(th);
			var compareMethod = th.getAttribute('compareMethod');
			if (compareMethod && compareMethod!="") compareMethod = SortTables_sortTypes[compareMethod];
			if (!compareMethod) continue;

			AddClass(th,'sortHead');
			th.style.cursor=document.getElementById?'pointer':'hand'; //should be 'pointer', which is W3C, IE6+ NS6+ compat, but not IE4/5.x
			th.setAttribute('origName',th.innerHTML);
			AttachEvent(th,'click',new Function('evt',"SortTables_SortRows("+tRef+","+sortBodyRef+","+thRef+","+j+","+compareMethod+")"), true );
			AttachEvent(th,'selectstart',PreventDefault);
			th.setAttribute('defaultSortDir',th.getAttribute('sortDescending') && th.getAttribute('sortDescending').toLowerCase()=="true"?'desc':'asc');
			if (th.getAttribute('defaultSort') && th.getAttribute('defaultSort').toLowerCase()=='true') SortTables_SortRows(tRef,sortBodyRef,thRef,j,compareMethod);
		}
	}
}

function SortTables_SortRows(tRef,tbodyRef,thRef,colNum,compareMethod){
	//DebugOut('SortTables_SortRows('+tRef+','+tbodyRef+','+thRef+','+colNum+','+compareMethod+')',1);
	var table = VirtualPointer.follow(tRef);
	var tbody = VirtualPointer.follow(tbodyRef);
	var th = VirtualPointer.follow(thRef);

	SortTables_GetVal = table.getAttribute('getValueFunction');
	if (SortTables_GetVal && SortTables_GetVal!="") SortTables_GetVal=window[SortTables_GetVal];
	else SortTables_GetVal=SortTables_TDVal;
	
	//sets some globals for use in the comparison function
	SortTables_sortAscending = HasClass(th,'sorted')?th.getAttribute('lastSortDir')=='desc':th.getAttribute('defaultSortDir')=='asc';
	SortTables_colToSort=colNum;
	SortTables_compareMethod=compareMethod;

	var allRowNodes=[],ct=0,childRows=[],childCt=0;
	var lastParent;
	for (var i=0,len=tbody.childNodes.length;i<len;i++){
		var tr = tbody.childNodes[i];
		if (tr.nodeType!=1) continue;
		if (tr.getAttribute("ischild") || HasClass(tr,'ischild')){
			if (!lastParent) continue;
			lastParent.childRows[lastParent.childRows.length]=tr;
			tr.rowParent = lastParent;
			childRows[childCt++]=tr;
		} else {
			lastParent = allRowNodes[ct++] = tr;
			lastParent.childRows=[];
		}
	}
	for (var i=0;i<childCt;i++) tbody.removeChild(childRows[i]);
	for (var i=0;i<ct;i++) tbody.removeChild(allRowNodes[i]);
	allRowNodes.sort(SortTables_CompareTDs);
	for (var i=0;i<ct;i++){
		var tr=allRowNodes[i];
		KillClass(tr,'row[01]');
		AddClass(tr,"row"+(i%2));
		for (var j=0,len2=tr.childRows.length;j<len2;j++){
			KillClass(tr.childRows[j],'row[01]');
			AddClass(tr.childRows[j],"row"+(i%2));
		}
		tbody.appendChild(tr);
	}
	for (var i=0;i<childCt;i++){
		var childRow = childRows[i];
		tbody.insertBefore(childRow,childRow.rowParent.nextSibling);
	}

	SortTables_SelectHead(th,table.getAttribute("showSortDirection") && table.getAttribute("showSortDirection").toLowerCase()=="true");
}

var floatRE = /-?(\d+(\.\d+)?|\.\d+)/;

function SortTables_CompareTDs(row1,row2){
	//DebugOut('SortTables_CompareTDs('+row1+','+row2+')',2);
	var row1val = SortTables_GetVal(row1,SortTables_colToSort);
	var row2val = SortTables_GetVal(row2,SortTables_colToSort);

	if (SortTables_compareMethod==SortTables_sortTypes.numeric) { isNaN(row1val*=1)?row1val=0:null; isNaN(row2val*=1)?row2val=0:null; }
	else if (SortTables_compareMethod==SortTables_sortTypes.price) { row1val=row1val.match(floatRE)[0]*1; row2val=row2val.match(floatRE)[0]*1; }
	else if (SortTables_compareMethod==SortTables_sortTypes.date) { row1val = new Date(row1val); row2val = new Date(row2val); if (isNaN(row1val)) row1val=0; if (isNaN(row2val)) row2val=0 }
	else if (SortTables_compareMethod==SortTables_sortTypes.time) {
		var rowPieces=row1val.split(":"),newVal=0;
		for (var i=0,len=rowPieces.length;i<len;i++)newVal+=(parseInt(rowPieces[i])*Math.pow(60,len-i-1));
		if (row1val.toLowerCase().indexOf("pm")!=-1)newVal+=12*Math.pow(60,len-1);
		row1val=newVal;
		
		rowPieces=row2val.split(":"),newVal=0;
		for (var i=0,len=rowPieces.length;i<len;i++)newVal+=(parseInt(rowPieces[i])*Math.pow(60,len-i-1));
		if (row2val.toLowerCase().indexOf("pm")!=-1)newVal+=12*Math.pow(60,len-1);
		row2val=newVal;		
	}else{
		row1val=row1val.toLowerCase();
		row2val=row2val.toLowerCase();
	}
	if (SortTables_sortAscending ? (row1val>row2val) : (row1val<row2val)) return 1;
	else if (SortTables_sortAscending ? (row1val<row2val) : (row1val>row2val)) return -1;
	else return 0;
}
function SortTables_TDVal(tr,colNum){
	//DebugOut('SortTables_TDVal('+tr+','+colNum+')',3);
	if (!tr) return null;
	var td = tr.childNodes[colNum];
	if (!td) return null;
	return td.childNodes[0].innerHTML.replace(/<\/?[^>]+>/g,'');
}

function SortTables_SelectHead(th,showDir){
	//DebugOut('SortTables_SelectHead('+th+')',2);
	if (SortTables_sortHead) SortTables_UnselectHead(SortTables_sortHead,showDir);
	AddClass(th,'sorted');
	if (showDir) th.innerHTML=th.getAttribute('origName')+' &#96'+(SortTables_sortAscending?5:6)+'0;';
	SortTables_sortHead=th;
	th.setAttribute('lastSortDir',SortTables_sortAscending?'asc':'desc');
}

function SortTables_UnselectHead(th,showDir){
	//DebugOut('SortTables_UnselectHead('+th+')',2);
	KillClass(th,'sorted');
	if (showDir) th.innerHTML=th.getAttribute('origName');
	SortTables_sortHead=null;
}

AttachEvent(window,'load',SortTables_Init,true);


/* Following is from virtualpointer.class.js
 * Copyright (c) 2002 Daniel Brockman.  All rights reserved.
 */
var VirtualPointer = {
  _pool:[null],
  create:function(o){ return this._pool.push(o)-1 },
  follow:function(id){ return this._pool[id] },
  resolve:function(id){ var result=this.follow(id); delete this._pool[id]; return result; }
};
