# TODO: ensure this constant is updated when/if a new language is added or prefix modified
LANGUAGE_PREFIXES = OJLanguage.map{ |lang| lang.prefix }.compact.join('|')

Ramaze::Route( 'language_name' ) do |url, request|
  url = case url
    when "/default.asp"    then "/"
    when "/property.asp"   then "/property/#{request['id']}"
    when "/object.asp"     then "/object/#{request['id']}"
    when "/objectused.asp" then "/object/#{request['id']}"
    when "/method.asp"     then "/method/#{request['id']}"
    when "/objects.asp"    then "/objects"    #TODO: look up on langID param
    when "/properties.asp" then "/properties" #TODO: look up on langID param
    when "/methods.asp"    then "/methods"    #TODO: look up on langID param
    when "/constants.asp"  then "/constants"  #TODO: look up on langID param
    when "/languages.asp"  then "/languages"
    when "/hierarchy.asp"  then "/hierarchy"
    when "/search.asp"     then "/search"
    else url
  end
  
  # Leading language path becomes param
  leading_prefix = %r{/(#{LANGUAGE_PREFIXES})\b(/.*)?}
  if url =~ leading_prefix
    request['lang_prefix'] = $1
    url = "/#{$2}"
  end
  url
end

class MainController < Controller
  before_all {
    @action    = Current.action.name.to_sym
    @languages = OJLanguage.order(:name).all
    @language  = @languages.find{ |l| l.prefix==request['lang_prefix'] } if request['lang_prefix']
  }
  
  helper :email_obfuscator
  helper :item_linkers
  
  def index
    @title = 'Welcome to ObjJob <span id="version">v2.0b</span>'
  end
  
  def languages
    @title = 'ObjJob Languages'
  end
  
  def hierarchy
    @title = "Hierarchy#{" for #{@language.linked_name}" if @language}"
    @lang_objs = @language ? @language.objects_dataset : OJObject.dataset
    @roots = @lang_objs.filter({:parent_id=>nil} | ~{:parent_id=>@lang_objs.select(:id)}).order(:name).all
  end
  
  def objects
    @title   = "Objects#{" in #{@language.linked_name}" if @language}"
    @objects = @language ? @language.objects_dataset : OJObject.dataset
    @objects = @objects.eager(:language).order(:name).all
  end
  
  def properties
    @title      = "Properties#{" in #{@language.linked_name}" if @language}"
    @properties = @language ? OJProperty.filter( :object_id=>@language.objects_dataset.select(:id) ) : OJProperty.dataset
    @properties = @properties.order( :name )
    render_partial :_property_list, :properties=>@properties, :id=>'properties', :title=>nil
  end
  
  def methods
    @title   = "Methods#{" in #{@language.linked_name}" if @language}"
    @methods = @language ? OJMethod.filter( :object_id=>@language.objects_dataset.select(:id) ) : OJMethod.dataset
    @methods = @methods.order( :name )
    render_partial :_method_list, :methods=>@methods, :id=>'methods', :title=>nil
  end
  
  def constants
    @title     = "Constants#{" in #{@language.linked_name}" if @language}"
    @constants = @language ? OJConstant.filter( :object_id=>@language.objects_dataset.select(:id) ) : OJConstant.dataset
    @constants = @constants.order( :name )
    render_partial :_constant_list, :constants=>@constants, :id=>'constants', :title=>nil
  end
  
  def object(object_id)
    @object = OJObject[object_id]
    @title  = "#{@object.name} in #{@language.linked_name}"
    @ancestors   = @object.ancestors
    @descendants = @object.descendants
    lineage_ids = @ancestors.map{ |o| o.pk } + [@object.pk]
    @my_properties  = OJProperty.filter :object_id => lineage_ids
    @my_methods     = OJMethod.filter   :object_id => lineage_ids
    @my_constants   = @object.constants_dataset

    @other_properties = OJProperty.filter :type_id        => object_id
    @other_methods    = OJMethod.filter   :return_type_id => object_id
    @other_arguments  = OJArgument.filter :type_id        => object_id
  end
  
  def _property_list
    @properties = @properties.eager( :object=>:language, :object_type=>:language ).order( :name ).all
  end
  
  def _method_list
    @methods = @methods.eager( :object=>:language, :return_type=>:language ).order( :name ).all
  end
  
  def _argument_list
    @arguments = @arguments.order( :name ).eager( :method=>{:object=>:language} ).all
  end

  def _constant_list
    @constants = @constants.order( :name ).eager( :object=>:language ).all
  end

  def property(property_id)
    @property = OJProperty[property_id]
    @title    = "#{object_link @property.object, (@property.instance && "my#{@property.object.name}") }.#{@property.name} in #{@property.object.language.linked_name}"
  end
  
  def method(method_id)
    @method   = OJMethod[method_id]
    @args     = @method.arguments_dataset.eager( :object_type=>:language ).all
    @arg_list = @args.map{ |arg|
      arg.required ? arg.name : "<span class='optional'>[ #{arg.name} ]</span>"
    }.join( ", " )
    @title    = "#{object_link @method.object, (@method.instance && "my#{@method.object.name}") }.#{@method.name}(#{ " <span class='arglist'>#{@arg_list}</span> " unless @arg_list.empty? }) in #{@method.object.language.linked_name}"
  end

  def search
    s = /#{request['q']}/i

    lang_id = request['langID']
    lang_id = nil if lang_id && lang_id.empty?
    @search_lang = OJLanguage[lang_id] if lang_id

    @title  = "Search of ObjJob for /#{request['q']}/i #{" in #{@search_lang.linked_name}" if @search_lang}"
    
    start_time = Time.now
    
    valid_o = @search_lang ? @search_lang.objects_dataset : OJObject.dataset
    valid_p = OJProperty.filter( :object_id=>valid_o.select(:id) )
    valid_c = OJConstant.filter( :object_id=>valid_o.select(:id) )
    valid_m = OJMethod.filter(   :object_id=>valid_o.select(:id) )
    valid_a = OJArgument.filter( :method_id=>valid_m.select(:id) )

    nulli = "NULL::integer"
    no_o  = "#{nulli} AS o_id"
    no_p  = "#{nulli} AS p_id"
    no_m  = "#{nulli} AS m_id"
    no_c  = "#{nulli} AS c_id"
    obji  = "object_id AS o_id"

    o_name = valid_o.grep( :name,        s ).select( "60 AS weight, id AS o_id, #{no_m}, #{no_p}, #{no_c}".lit )
    o_view = valid_o.grep( :overview,    s ).select( "30 AS weight, id AS o_id, #{no_m}, #{no_p}, #{no_c}".lit )
    o_desc = valid_o.grep( :description, s ).select( "10 AS weight, id AS o_id, #{no_m}, #{no_p}, #{no_c}".lit )

    p_name = valid_p.grep( :name,        s ).select( "60 AS weight, #{obji}, #{no_m}, id AS p_id, #{no_c}".lit )
    p_view = valid_p.grep( :overview,    s ).select( "30 AS weight, #{obji}, #{no_m}, id AS p_id, #{no_c}".lit )
    p_desc = valid_p.grep( :description, s ).select( "10 AS weight, #{obji}, #{no_m}, id AS p_id, #{no_c}".lit )

    c_name = valid_c.grep( :name,        s ).select( "70 AS weight, #{obji}, #{no_m}, #{no_p}, id AS c_id".lit )
    c_view = valid_c.grep( :overview,    s ).select( "30 AS weight, #{obji}, #{no_m}, #{no_p}, id AS c_id".lit )

    m_name = valid_m.grep( :name,        s ).select( "60 AS weight, #{obji}, id AS m_id, #{no_p}, #{no_c}".lit )
    m_view = valid_m.grep( :overview,    s ).select( "20 AS weight, #{obji}, id AS m_id, #{no_p}, #{no_c}".lit )
    m_desc = valid_m.grep( :description, s ).select( " 5 AS weight, #{obji}, id AS m_id, #{no_p}, #{no_c}".lit )
    a_name = valid_m.filter( :id => valid_a.grep( :name,     s ).select(:method_id) ).select( "10 AS weight, #{obji}, id AS m_id, #{no_p}, #{no_c}".lit )
    a_view = valid_m.filter( :id => valid_a.grep( :overview, s ).select(:method_id) ).select( " 5 AS weight, #{obji}, id AS m_id, #{no_p}, #{no_c}".lit )

    weights_on_ids = [ o_name, o_view, o_desc,
                     m_name, m_view, m_desc, a_name, a_view,
                     p_name, p_view, p_desc,
                     c_name, c_view ].inject{ |all,ds| all.union(ds) }

    weighted_ids   = weights_on_ids.from_self
                     .select( "SUM(weight) AS weight".lit, :o_id, :m_id, :p_id, :c_id )
                     .group_by( :o_id, :m_id, :p_id, :c_id )

    @results = weighted_ids.from_self
              .inner_join(      :objects,    :id=>:o_id )
              .inner_join(      :languages,  :id=>:language_id )
              .left_outer_join( :methods,    :id=>:t1__m_id )
              .left_outer_join( :properties, :id=>:t1__p_id )
              .left_outer_join( :constants,  :id=>:t1__c_id )
              .select(
                :weight,
                :language_id, :languages__name___l_name,  :languages__prefix___prefix,
                :o_id, :objects__name___o_name,    :objects__overview___o_overview,
                :m_id, :methods__name___m_name,    :methods__overview___m_overview,
                :p_id, :properties__name___p_name, :properties__overview___p_overview,
                :c_id, :constants__name___c_name,  :constants__overview___c_overview
              )
              .order( :weight.desc, :o_name, :m_name, :p_name, :c_name )
              .all
    
    @search_time = Time.now - start_time
  end
end