$: << ".." # ...because init.rb is used to being called from one directory up
$: << "migration_libs" # ...so TableTransloader can load some siblings

require 'init.rb'
require 'migration_libs/TableTransloader.rb'
require 'logger'

$SHOW_PROGRESS = true

class OJLanguage
  extend Phrogz::TableTransloader
  transload_from( 'ObjJob_Languages.csv', :langID ) do |old|
    #"langID","langName","url","prefix","copyright","directoryName","keywords","overview"    
    {
      :id        => old[:langID],
      :name      => old[:langName],
      :url       => old[:url],
      :prefix    => old[:prefix],
      :copyright => old[:copyright],
      :directory => old[:directoryName],
      :keywords  => old[:keywords],
      :overview  => old[:overview]
    }
  end
end

class OJObject
  extend Phrogz::TableTransloader
  transload_from( 'ObjJob_Objects.csv', :objID ) do |old|
    #"objID","langID","objName","overview","description","url","keywords"
    {
      :id          => old[:objID],
      :language_id => old[:langID],
      :parent_id   => nil,
      :name        => old[:objName],
      :overview    => old[:overview],
      :description => old[:description],
      :keywords    => old[:keywords],
      :url         => old[:url]
    }
  end
end

class OJProperty
  extend Phrogz::TableTransloader
  transload_from( 'ObjJob_Properties.csv', :id ) do |old|
    #"id","objID","propName","objTypeID","readOnly","overview","description","keywords","isClassProperty"
    {
      :id          => old[:id],
      :object_id   => old[:objID], 
      :type_id     => old[:objTypeID], 
      :name        => old[:propName], 
      :overview    => old[:overview], 
      :description => old[:description], 
      :keywords    => old[:keywords], 
      :read_only   => old[:readOnly], 
      :instance    => !old[:isClassProperty]
    }
  end
end 
 
class OJMethod
  extend Phrogz::TableTransloader
  transload_from( 'ObjJob_Methods.csv', :id ) do |old|
    #"id","objID","methodName","returnTypeID","overview","description","keywords","isClassMethod"
    {
      :id             => old[:id],
      :object_id      => old[:objID],
      :return_type_id => old[:returnTypeID],
      :name           => old[:methodName],
      :overview       => old[:overview],
      :description    => old[:description],
      :keywords       => old[:keywords],
      :instance       => !old[:isClassMethod]
    }
  end
end 
 
class OJArgument
  extend Phrogz::TableTransloader
  transload_from( 'ObjJob_MethodArguments.csv', :id ) do |old|
    #"id","methodID","argNum","argName","argTypeID","reqd","overview"
    {
      :id        => old[:id],
      :method_id => old[:methodID],
      :type_id   => old[:argTypeID],
      :name      => old[:argName],
      :overview  => old[:overview],
      :sort      => old[:argNum],
      :required  => old[:reqd]
    }
  end
end

class OJConstant
  extend Phrogz::TableTransloader
  transload_from( 'ObjJob_Constants.csv', :id ) do |old|
    #"id","constName","objID","constTypeID","constValue","overview"
    {
      :id        => old[:id],
      :object_id => old[:objID],  
      :type_id   => old[:constTypeID],  
      :name      => old[:constName],  
      :overview  => old[:overview],  
      :value     => old[:constValue]
    }
  end
end 

Phrogz::TableTransloader.transload_all

Phrogz::RoughCSV.load( 'ObjJob_Hier.csv' ).each do |old_row|
  new_child  = OJObject.pk_after_transload[ old_row[:childID ] ]
  new_parent = OJObject.pk_after_transload[ old_row[:parentID] ]
  DB.execute "UPDATE objects SET parent_id=#{new_parent} WHERE id=#{new_child}"
  #child = OJObject[new_child]
  #child.update( :parent_id=>new_parent )
  #child.parent_id = new_parent; child.save
end