module Phrogz; end
module Phrogz::RoughCSV
  def self.load( file_name )
    require 'time' # for parsing
    csv = IO.read( file_name )
    
    # Silly DOS line endings
    csv.gsub! "\r\n", "\n"
    
    # Rough cleaning to match the simple regex
    :go while csv.gsub! /^,/, 'NULL,'
    :go while csv.gsub! ',,', ',NULL,'
    :go while csv.gsub! ',"",', ',NULL,'
    :go while csv.gsub! ",\"\"\n", ",NULL\n"

    # first line must be columns
    header = csv[ /\A.+?\n/ ]
    csv = csv[ header.length..-1 ]
    columns = header.scan( /"([^"]+)"|([^,\n]+)/ ).flatten.compact.map{ |col| col.to_sym }
    rows = []
    rows << current_row = {}
    col_index = -1

    csv.scan( /([^",\n]+|".*?[^"](?:"")*")(,|\n)/m ) do |item, sep|
      col = columns[ col_index+=1 ]

      # Guess at native Ruby value types
      current_row[ col ] = case
        when item[0]==?"
          item[1..-2].gsub('""','"')
        when item =~ /^-?\d+$/
          item.to_i
        when item =~ /^-?\d+\.\d+$/
          item.to_f
        when item == "True"
          true
        when item == "False"
          false
        when item == 'NULL'
          nil
        when item =~ /(?:20|19)\d{2}-\d{2}-\d{2}\b/
          Time.parse(item)
        else
          puts "Treating #{item.inspect} as String value..."
          item
      end

      if sep == "\n"
        rows << current_row = {}
        col_index = -1
      end
    end

    # Throw out the final, empty row
    rows.pop if rows.last.empty?
    rows
  end
end
