class OJObject < Sequel::Model( :objects )
  many_to_one :language,       :class=>:OJLanguage, :key=>:language_id
  many_to_one :parent_object,  :class=>:OJObject,   :key=>:parent_id

  # The eager call is a hack needed if you remove the object_id methods from these classes
  one_to_many :properties,     :class=>:OJProperty, :key=>:object_id#, :eager=>:object
  one_to_many :methods,        :class=>:OJMethod,   :key=>:object_id#, :eager=>:object
  one_to_many :constants,      :class=>:OJConstant, :key=>:object_id
  one_to_many :constant_types, :class=>:OJConstant, :key=>:object_id
  one_to_many :property_types, :class=>:OJProperty, :key=>:type_id
  one_to_many :argument_types, :class=>:OJArgument, :key=>:type_id
  one_to_many :return_types,   :class=>:OJMethod,   :key=>:return_type_id
  one_to_many :child_objects,  :class=>:OJObject,   :key=>:parent_id
  
  def ancestors
    chain  = []
    walker = self.parent_object
    while walker
      # TODO: check for circular chains here
      chain << walker
      walker = walker.parent_object
    end
    chain.reverse
  end
  
  def descendants
    all = []
    # TODO: check for circular chains here
    all.concat( child_objects )
    child_objects.each{ |child| all << child.descendants }
  end
end

class OJProperty < Sequel::Model( :properties )
  many_to_one :object,      :class=>:OJObject, :key=>:object_id
  many_to_one :object_type, :class=>:OJObject, :key=>:type_id 
  
  # Because Object#object_id exists
  def object_id; self[:object_id]; end
end

class OJMethod < Sequel::Model( :methods )
  many_to_one :object,      :class=>:OJObject,   :key=>:object_id
  many_to_one :return_type, :class=>:OJObject,   :key=>:return_type_id
  
  one_to_many :arguments,   :class=>:OJArgument, :key=>:method_id, :order=>:sort

  # Because Object#object_id exists
  def object_id; self[:object_id]; end
end

class OJArgument < Sequel::Model( :arguments )
  many_to_one :method,      :class=>:OJMethod, :key=>:method_id
  many_to_one :object_type, :class=>:OJObject, :key=>:type_id 
end
