DROP TABLE IF EXISTS languages;
CREATE TABLE languages (
	id         SMALLINT PRIMARY KEY,
  name       VARCHAR(50)   NOT NULL,
  url        VARCHAR(200)      NULL,
  prefix     VARCHAR(20)       NULL,
  copyright  VARCHAR(1000)     NULL,
  directory  VARCHAR(50)   NOT NULL,
  keywords   VARCHAR(50)       NULL,
  overview   VARCHAR(500)      NULL,
  created_on TIMESTAMP     NOT NULL DEFAULT now()
);

DROP TABLE IF EXISTS objects;
CREATE TABLE objects ( 
	id          SERIAL PRIMARY KEY,
	language_id INT           NOT NULL REFERENCES languages(id),
  parent_id   INT               NULL REFERENCES objects(id),
  name        VARCHAR(50)   NOT NULL,
  overview    VARCHAR(200)      NULL,
  description VARCHAR(6000)     NULL,
  keywords    VARCHAR(50)       NULL,
  url         VARCHAR(200)      NULL,  
  created_on  TIMESTAMP     NOT NULL DEFAULT now()
);

DROP TABLE IF EXISTS constants;
CREATE TABLE constants (
	id         SERIAL PRIMARY KEY,
	object_id  INT           NOT NULL REFERENCES objects(id),
  type_id    INT           NOT NULL REFERENCES objects(id),
  name       VARCHAR(50)   NOT NULL,
  overview   VARCHAR(200)      NULL,
  value      VARCHAR(40)       NULL,
  created_on TIMESTAMP     NOT NULL DEFAULT now()
);

DROP TABLE IF EXISTS properties;
CREATE TABLE properties ( 
	id          SERIAL PRIMARY KEY,
	object_id   INT           NOT NULL REFERENCES objects(id),
  type_id     INT           NOT NULL REFERENCES objects(id),
  name        VARCHAR(50)   NOT NULL,
  overview    VARCHAR(200)      NULL,
  description VARCHAR(6000)     NULL,
  keywords    VARCHAR(50)       NULL,
  read_only   BOOLEAN       NOT NULL DEFAULT FALSE,
  instance    BOOLEAN       NOT NULL DEFAULT TRUE,
  created_on  TIMESTAMP     NOT NULL DEFAULT now()
);

DROP TABLE IF EXISTS methods;
CREATE TABLE methods (
	id              SERIAL PRIMARY KEY,
	object_id       INT           NOT NULL REFERENCES objects(id),
  return_type_id  INT           NOT NULL REFERENCES objects(id),
  name            VARCHAR(50)   NOT NULL,
  overview        VARCHAR(200)      NULL,
  description     VARCHAR(6000)     NULL,
  keywords        VARCHAR(50)       NULL,
  instance        BOOLEAN       NOT NULL DEFAULT TRUE,
  created_on      TIMESTAMP     NOT NULL DEFAULT now()
);

DROP TABLE IF EXISTS arguments;
CREATE TABLE arguments (
	id         SERIAL PRIMARY KEY,
	method_id  INT           NOT NULL REFERENCES methods(id),
  type_id    INT           NOT NULL REFERENCES objects(id),
  name       VARCHAR(50)   NOT NULL,
  overview   VARCHAR(200)      NULL,
  sort       SMALLINT      NOT NULL DEFAULT 1,
  required   BOOLEAN       NOT NULL DEFAULT TRUE
);

DROP TABLE IF EXISTS users;
CREATE TABLE users ( 
	id         SERIAL PRIMARY KEY,
	login      VARCHAR(50)   NOT NULL UNIQUE,
  email      VARCHAR(50)       NULL,
  name       VARCHAR(50)       NULL,
  pass_hash  VARCHAR(50)   NOT NULL,
  disabled   BOOLEAN       NOT NULL DEFAULT FALSE,
  created_on TIMESTAMP     NOT NULL DEFAULT now()
);

/*
DROP TABLE IF EXISTS comment_types;
CREATE TABLE comment_types 
CREATE TABLE [dbo].[t_Comment_CommentTypes] (
	[id] [int] IDENTITY (1, 1) NOT NULL ,
	[commentID] [int] NOT NULL ,
	[commentTypeID] [tinyint] NOT NULL 
) ON [PRIMARY]
CREATE TABLE [dbo].[t_Comments] (
	[commentID] [int] IDENTITY (1, 1) NOT NULL ,
	[objectID] [int] NOT NULL ,
	[propertyID] [int] NULL ,
	[methodID] [int] NULL ,
	[authorID] [int] NOT NULL ,
	[authorVerified] [bit] NOT NULL ,
	[subject] [varchar] (100) COLLATE SQL_Latin1_General_CP1_CI_AS NULL ,
	[comment] [varchar] (6000) COLLATE SQL_Latin1_General_CP1_CI_AS NULL ,
	[commentDate] [smalldatetime] NOT NULL 
) ON [PRIMARY]
CREATE TABLE [dbo].[t_Hier] (
	[id] [int] IDENTITY (1, 1) NOT NULL ,
	[parentID] [int] NOT NULL ,
	[childID] [int] NOT NULL 
) ON [PRIMARY]
CREATE TABLE [dbo].[t_Log] (
	[logID] [int] IDENTITY (1, 1) NOT NULL ,
	[logTime] [datetime] NOT NULL ,
	[eventTypeID] [tinyint] NOT NULL ,
	[ipAddress] [varchar] (15) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL ,
	[userID] [int] NULL ,
	[langID] [int] NULL ,
	[objID] [int] NULL ,
	[constID] [int] NULL ,
	[propID] [int] NULL ,
	[methID] [int] NULL ,
	[argID] [int] NULL ,
	[data] [varchar] (100) COLLATE SQL_Latin1_General_CP1_CI_AS NULL 
) ON [PRIMARY]
CREATE TABLE [dbo].[t_LogEventTypes] (
	[eventTypeID] [tinyint] NOT NULL ,
	[eventType] [varchar] (20) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL 
) ON [PRIMARY]
*/